/*
** iplog_lockfile.c - iplog lockfile management.
** Copyright (C) 1999 Odin <odin@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
*/

#include <config.h>

#define _GNU_SOURCE
#include <unistd.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <sys/stat.h>
#include <sys/file.h>

#ifdef HAVE_PATHS_H
#include <paths.h>
#endif

#include <iplog.h>

#ifndef _PATH_VARRUN
#	define LOCKFILE "/etc/iplog.pid"
#else
#	define LOCKFILE (_PATH_VARRUN "iplog.pid")
#endif

void write_lockfile(void) {
	int fd;
	struct flock fl;

	memset(&fl, 0, sizeof(fl));

	fd = open(LOCKFILE, O_RDWR | O_CREAT | O_TRUNC | O_EXCL, 0644);

	if (fd == -1) {
		if (errno != EEXIST)
			fatal("Cannot open lockfile \"%s\": %s", LOCKFILE, strerror(errno));

		fd = open(LOCKFILE, O_RDONLY);
		if (fd == -1)
			fatal("Cannot open existing lockfile.");

		if (fcntl(fd, F_GETLK, &fl) == -1 || fl.l_type == F_UNLCK) {
			if (unlink(LOCKFILE) != 0)
				fatal("Could not remove stale lockfile \"%s\"", LOCKFILE);
		} else
			fatal("iplog is already running (pid: %d)", fl.l_pid);

		close(fd);

		fd = open(LOCKFILE, O_RDWR | O_CREAT | O_TRUNC, 0644);
		if (fd == -1)
			fatal("Cannot create lockfile \"%s\": %s",
				LOCKFILE, strerror(errno));
	}

	dprintf(fd, "%d\n", getpid());

	fl.l_type = F_WRLCK;
	fl.l_whence = SEEK_SET;
	fl.l_start = 0;
	fl.l_len = 0;

	if (fcntl(fd, F_SETLK, &fl) == -1)
		fatal("Lockfile \"%s\" could not be locked: %s", LOCKFILE,
			strerror(errno));
}

void kill_iplog(int sig) {
	struct stat st;
	int fd;
	u_char buf[16];
	ssize_t len;
	pid_t pid;

	if (lstat(LOCKFILE, &st) != 0)
		fatal("iplog is not running.");
	
	fd = open(LOCKFILE, O_RDONLY);
	if (fd == -1)
		fatal("Can't open pid file: %s", strerror(errno));

	len = read(fd, buf, sizeof(buf));
	if (len < 1)
		fatal("Error reading pid.");

	buf[len] = '\0';
	pid = strtoul(buf, NULL, 10);

	if (kill(pid, sig) != 0) {
		unlink(LOCKFILE);
		fatal("Can't send process signal %d: %s", sig, strerror(errno));
	}

	if (sig != 1 && unlink(LOCKFILE) != 0)
		fatal("Can't unlink lockfile: %s", strerror(errno));

	exit(0);
}

/* vim:ts=4:sw=8:tw=0 */
